import streamlit as st
st.set_page_config(layout="wide", initial_sidebar_state="expanded")

from shrinkit.ml.imputation import *
from shrinkit.ml.filtration import *
from shrinkit.ml.machine_learning import *
from shrinkit.ml.encoding import *
from shrinkit.ml.normalization import *
from shrinkit.ml.evaluation import *


from sklearn.metrics import mean_absolute_error, mean_squared_error, r2_score
from sklearn.metrics import mean_squared_log_error
from sklearn.metrics import accuracy_score, precision_score, recall_score, f1_score, roc_auc_score
import numpy as np
import pandas as pd


class CustomEvaluation:
    def __init__(self, y_test, y_preds, n_train_cols=0, category=None):
        self.y_test = y_test
        self.y_preds = y_preds
        self.category = category
        self.n_train_cols = n_train_cols


    def evaluate(self):
        if self.category == "regression":
            metrics = ['MAE', 'MSE', 'RMSE', 'RMSLE', 'R2', 'adjusted_R2']
            results = pd.DataFrame(index=metrics, columns=self.y_preds.keys())

            for model_name, y_pred in self.y_preds.items():
                mae = mean_absolute_error(self.y_test, y_pred)
                mse = mean_squared_error(self.y_test, y_pred)
                rmse = np.sqrt(mse)
                rmsle = np.sqrt(mean_squared_log_error(self.y_test, y_pred))
                r2 = r2_score(self.y_test, y_pred)
                n = len(self.y_test)
                k = self.n_train_cols
                adjusted_r2 = 1 - (1 - r2) * (n - 1) / (n - k - 1)

                results.loc['MAE', model_name] = mae
                results.loc['MSE', model_name] = mse
                results.loc['RMSE', model_name] = rmse
                results.loc['RMSLE', model_name] = rmsle
                results.loc['R2', model_name] = r2
                results.loc['adjusted_R2', model_name] = adjusted_r2
            results = results.transpose()
            results.index = results.index.set_names(['Models Name    '])
            return results


        elif self.category == "classification":
            metrics = ['Accuracy', 'Precision', 'Recall', 'F1']
            results = pd.DataFrame(index=metrics, columns=self.y_preds.keys())

            for model_name, y_pred in self.y_preds.items():
                accuracy = accuracy_score(self.y_test, y_pred)
                precision = precision_score(self.y_test, y_pred, average='macro')
                recall = recall_score(self.y_test, y_pred, average='macro')
                f1 = f1_score(self.y_test, y_pred, average='macro')
                # auc = roc_auc_score(self.y_test, y_pred)

                results.loc['Accuracy', model_name] = accuracy
                results.loc['Precision', model_name] = precision
                results.loc['Recall', model_name] = recall
                results.loc['F1', model_name] = f1
                # results.loc['ROC_AUC', model_name] = auc

            results.index = results.index.set_names(['Models Name'])
            return results









from sklearn.preprocessing import StandardScaler
import pandas as pd

class CustomNormalizer:
    def __init__(self, X, status):
        self.X = X
        self.status = status

    def normalize(self):
        normalizer = StandardScaler()
        X_scaled = pd.DataFrame(normalizer.fit_transform(self.X), columns=self.X.columns)
        return X_scaled




from shrinkit.main import *
# import os
# import sys
# import streamlit.web.cli as stcli


# this_dir = os.path.dirname(os.path.abspath(__file__))
# os.chdir(this_dir)
# print(this_dir)
# sys.path.append(this_dir)
# sys.argv = ["streamlit", "run", "main.py", "--global.developmentMode=false", "--browser.gatherUsageStats=false"]
# sys.exit(stcli.main())


import streamlit as st

import pandas as pd
import numpy as np
from sklearn.model_selection import train_test_split
from ydata_profiling import ProfileReport
from streamlit_ydata_profiling import st_profile_report


from shrinkit.ml.imputation import CustomImputer
from shrinkit.ml.filtration import DataFiltration
from shrinkit.ml.machine_learning import MLModeling
from shrinkit.ml.encoding import CustomEncoding
from shrinkit.ml.normalization import CustomNormalizer
from shrinkit.ml.evaluation import CustomEvaluation

import warnings
warnings.filterwarnings("ignore")

class Shrinkit():
    def __init__(self):
        st.markdown("# Shrink-it Playground! 🚀")
        uploaded_file = st.file_uploader("Choose a CSV file", type="csv")
        if uploaded_file is not None:
            self.data = pd.read_csv(uploaded_file)
            self.is_data_available = True
        else:
            self.is_data_available = False

        

    def run(self):
        status = st.markdown("#### Status: Reading Dataset...")
        st.divider()
        st.sidebar.title('Shrink-it Config ⚙️')
        st.sidebar.write("Choose the configurations below:")
        st.sidebar.divider()
        # ==============================================================
        # =================== READ DATA AND PROCESS IT =================
        # ==============================================================
        st.sidebar.title('Reading Data:')
        c1, c2 = st.sidebar.columns(2)
        show_original_data = c1.toggle(label='Show', key="show original dataset", value=False)
        show_data_profiling = c2.toggle(label='Analysis', key="show profiler results", value=False)
        st.sidebar.write("Note: Keep the Analysis OFF if you are not analyzing the data. It is compute intensive!")
        if show_original_data:
            try:
                st.markdown(f"#### Original Data: {self.data.shape}")
                st.write(self.data.head())
                st.divider()
                self.is_data_available = True
            except:
                self.is_data_available = False

        if self.is_data_available and show_data_profiling:
            st.markdown("### Data Analysis Results & Visualizations:")
            profile = ProfileReport(self.data, title="Pandas Profiling Report", minimal=True, explorative=True)
            st_profile_report(profile, navbar=True)
        else:
            status.markdown("#### Status: Please upload dataset...")


        if self.is_data_available:
            # ===============================================================
            # ========================= DATA FILTRATION =====================
            # ===============================================================
            st.sidebar.divider()
            st.sidebar.title('Understanding Data:')
            show_filter_results = st.sidebar.toggle(label='Show', key="show data filtered results")
            filter = DataFiltration(self.data)
            X, y, id_column = filter.filter_data(status)
            if show_filter_results:
                st.markdown("### Filtered Data:")
                c1, c2 = st.columns(2)
                c1.write(f"Independent Features: {X.shape}")
                c1.write(X.head(5))
                c2.write(f"Dependent Features: {y.shape}")
                c2.write(y.head(5))
                st.divider()

            print(f"ID COL: {id_column}")
            if id_column and id_column != "none":
                X = X.drop([id_column], axis=1)
            # ===============================================================
            # =================== MISSING VALUES IMPUTATION =================
            # ===============================================================
            st.sidebar.divider()
            st.sidebar.title('Missing Imputation:')
            show_imputation_results = st.sidebar.toggle(label='Show', key="missing values show")
            imputer = CustomImputer()
            X = imputer.execute_missing_value_imputation(X, status)
            if show_imputation_results:
                st.markdown(f"### Imputed Dataset: {X.shape}")
                st.write(X)


            # ===============================================================
            # =========================== ENCODING DATA ====================
            # ===============================================================
            st.sidebar.divider()
            st.sidebar.title('Encoding Data:')
            show_encoded_results = st.sidebar.toggle(label='Show', key="encoded data show")
            encoder = CustomEncoding(X, status)
            X = encoder.encode()
            if show_encoded_results:
                st.markdown(f"### Encoded Dataset: {X.shape}")
                st.write(X)

            # ===============================================================
            # ========================= NORMALIZE DATA ====================
            # ===============================================================
            st.sidebar.divider()
            st.sidebar.title('Normalize Data:')
            show_normalized_results = st.sidebar.toggle(label='Show', key="normalized data show")
            normalizer = CustomNormalizer(X, status)
            X = normalizer.normalize()
            if show_normalized_results:
                st.markdown(f"### Normalized Dataset: {X.shape}")
                st.write(X)

            # ===============================================================
            # ========================= DATA SPLITTING ======================
            # ===============================================================
            st.sidebar.divider()
            st.sidebar.title('Splitting Data:')
            train_split = st.sidebar.slider('Training data (%):', 1, 99, 75)
            test_size = (100-train_split)/100
            print(f"Test data size: {test_size}")
            X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=(100-train_split)/100, random_state=42)

            # ===============================================================
            # ========================= MACHINE LEARNING ====================
            # ===============================================================
            st.sidebar.divider()
            st.sidebar.title('Machine Learning:')
            show_ml_results = st.sidebar.toggle(label='Show Training Results', key="show training metric results")
            show_predictions = st.sidebar.toggle(label='Show Predictions', key="show predictions data")
            ml_modeling = MLModeling(X_train, X_test, y_train, y_test)
            predictions_dict, cateogry = ml_modeling.compute_ML()

            # ======================= EVALUATION METRICS ====================
            eval = CustomEvaluation(y_test, predictions_dict, X_train.shape[1], cateogry)
            matrix_table = eval.evaluate()
            if show_ml_results:
                st.markdown(f"### Evaluation Metrics Table: ({cateogry.capitalize()} Models)")
                st.write(matrix_table)
                st.divider()

            if show_predictions:
                st.markdown(f"### Predictions: ({len(predictions_dict)})")
                st.write(predictions_dict)
                st.divider()
            


if __name__ == "__main__":
    st.divider()
    skt = Shrinkit()
    skt.run()



import streamlit as st
import pandas as pd

class DataFiltration:
    def __init__(self, data):
        self.data = data

    def filter_data(self, status):
        id_column = st.sidebar.selectbox('Choose ID Column', ['none'] + list(self.data.columns))
        target_column = st.sidebar.selectbox('Choose Target Column', self.data.columns)
        print(target_column)

        if target_column:
            status.markdown("#### Status: Target Column Chosen.")
            X = self.data.drop([target_column], axis=1)
            y = self.data.loc[:, [target_column]]
        else:
            status.markdown("#### Status: Choosing Target Columns...")

        
        return X, y, id_column


import streamlit as st
from sklearn.svm import SVC, SVR
from sklearn.neighbors import KNeighborsClassifier, KNeighborsRegressor
from sklearn.ensemble import RandomForestClassifier, RandomForestRegressor
from sklearn.linear_model import LogisticRegression, LinearRegression, Ridge, Lasso, \
    ElasticNet, OrthogonalMatchingPursuit, BayesianRidge, ARDRegression, HuberRegressor, \
        QuantileRegressor, RANSACRegressor, TheilSenRegressor, PoissonRegressor, GammaRegressor

from sklearn.tree import DecisionTreeClassifier, DecisionTreeRegressor
from sklearn.neural_network import MLPClassifier

class MLModeling:
    def __init__(self, X_train, X_test, y_train, y_test):
        self.X_train = X_train
        self.X_test = X_test
        self.y_train = y_train
        self.y_test = y_test


    def compute_ML(self):
        data_type = st.sidebar.selectbox('ML Model Type', ['none', 'regression', 'classification'])
        if data_type == "regression":
            preds = self.execute_regression_pipeline()
        elif data_type == "classification":
            preds = self.execute_classification_pipeline()
        else:
            preds = []
        return preds, data_type
    
    
    def train_model(self, selected_models, models):
        preds = dict()
        print(selected_models)
        for key in selected_models:
            model = models[key]
            model.fit(self.X_train, self.y_train)
            y_pred = model.predict(self.X_test)
            preds[key] = y_pred
        return preds


    def execute_regression_pipeline(self):
        models = ['Linear Regression', 'Ridge', 'Lasso', 'ElasticNet', 'SVM', 'Random Forest Regressor', \
                'Decision Tree Regressor', 'Bayesian Ridge', 'Poisson Regressor', \
                'Gamma Regressor', 'ARD Regression', 'Huber Regressor', \
                'RANSAC Regressor', 'TheilSen Regressor', 'Orthogonal Matching Pursuit']
        
        st.sidebar.markdown("#### Select Regression Models:")
        select_all_toggle = st.sidebar.toggle(label="Select All", key="Select all regression models")
        selected_models = []

        if select_all_toggle:
            selected_models = models
            for model in models:
                st.sidebar.checkbox(model, value=True) 
        else:
            for model in models:
                selected = st.sidebar.checkbox(model)
                if selected:
                    selected_models.append(model)
    
        models = {
                'Linear Regression': LinearRegression(),
                'Ridge' : Ridge(),
                'Lasso' : Lasso(),
                'ElasticNet': ElasticNet(),
                'Decision Tree Regressor': DecisionTreeRegressor(),
                'Bayesian Ridge': BayesianRidge(),
                'Poisson Regressor': PoissonRegressor(),
                'Gamma Regressor': GammaRegressor(),
                'ARD Regression': ARDRegression(),
                'Huber Regressor': HuberRegressor(),
                'RANSAC Regressor': RANSACRegressor(),
                'TheilSen Regressor': TheilSenRegressor(),
                'Orthogonal Matching Pursuit': OrthogonalMatchingPursuit(),
                'SVM': SVR(),
                'Random Forest Regressor': RandomForestRegressor()
            }
        return self.train_model(selected_models, models)


    def execute_classification_pipeline(self):
        # selected_models = st.sidebar.multiselect('Classification Models', ['Logistic Regression', 'SVM', 'KNN', "Random Forest"])
        models = ['Logistic Regression', 'Support Vector', 'KNN', \
                'Decision Tree Classifier', 'Random Forest Classifier', 'MLP Classifier']
        
        st.sidebar.markdown("#### Select Classification Models:")
        select_all_toggle = st.sidebar.toggle(label="Select All", key="Select all classification models")
        selected_models = []

        if select_all_toggle:
            selected_models = models
            for model in models:
                st.sidebar.checkbox(model, value=True) 
        else:
            for model in models:
                selected = st.sidebar.checkbox(model)
                if selected:
                    selected_models.append(model)
        models = {
                'Logistic Regression': LogisticRegression(),
                'Support Vector': SVC(),
                'KNN': KNeighborsClassifier(),
                'Decision Tree Classifier': DecisionTreeClassifier(),
                'Random Forest Classifier': RandomForestClassifier(),
                'MLP Classifier': MLPClassifier()
            }
        return self.train_model(selected_models, models)
        
        



import pandas as pd

class CustomEncoding:
    def __init__(self, X, status):
        self.X = X
        self.status = status

    def encode(self):
        self.X = pd.get_dummies(self.X)
        return self.X


import sys
import os
import streamlit.web.cli as stcli
import subprocess


class ShrinkitRunner:
    def __init__(self):
        pass

    def streamlit_run(self):
        this_dir = os.path.dirname(os.path.abspath(__file__))
        os.chdir(this_dir)
        sys.path.append(this_dir)
        sys.argv = ["streamlit", "run", "main.py", "--global.developmentMode=false", "--browser.gatherUsageStats=false"]
        sys.exit(stcli.main())

    # def run(self):
        # sys.argv = ["streamlit", "run", "./shrinkit/main.py"]
        # sys.exit(stcli.main())

        # current_dir = os.path.dirname(os.path.abspath(__file__))
        # print(current_dir)
        # main_file_path = os.path.join(current_dir, 'main.py')
        # sys.argv = ["streamlit", "run", main_file_path]
        # sys.exit(stcli.main())
        
        # current_dir = os.path.dirname(os.path.abspath(__file__))
        # subprocess.Popen(f'sh {current_dir}/top_run.sh', shell=True)



from sklearn.impute import SimpleImputer
import streamlit as st
import pandas as pd
import numpy as np

class CustomImputer:
    def __init__(self):
        pass

    def execute_missing_value_imputation(self, X, status):
        
        c1, c2 = st.sidebar.columns(2)
        imputation_strategy_num = c1.selectbox('Numerical', ['none', 'mean', 'median', 'most_frequent', 'constant'])
        imputation_strategy_cat = c2.selectbox('Categorical', ['none', 'most_frequent', 'constant'])

        if imputation_strategy_num == 'constant':
            const_fill_num = c1.text_input('Numerical Fill Value', -1)
        else:
            const_fill_num = None
        
        if imputation_strategy_cat == 'constant':
            const_fill_cat = c2.text_input('Categorical Fill Value', "missing")
        else:
            const_fill_cat = None

        num_data = X.select_dtypes(include='number')
        print(f"Num cols shape: {len(num_data.columns)}")

        cat_cols = [col for col in X.columns if col not in num_data.columns]
        print(f"Cat cols shape: {len(cat_cols)}")
        cat_data = X.loc[:, cat_cols]

        if imputation_strategy_num != "none":
            status.markdown("#### Status: Missing Values Imputed.")
            if const_fill_num:
                imp_num = SimpleImputer(missing_values=np.nan, strategy=imputation_strategy_num, fill_value=int(const_fill_num))
            else:
                imp_num = SimpleImputer(missing_values=np.nan, strategy=imputation_strategy_num)
            num_data_imputed = pd.DataFrame(imp_num.fit_transform(num_data), columns=num_data.columns)
        else:
            num_data_imputed = num_data
            status.markdown("#### Status: Imputing missing values...")
            
        if imputation_strategy_cat != "none":
            status.markdown("#### Status: Missing Values Imputed.")
            imp_cat = SimpleImputer(missing_values=np.nan, strategy=imputation_strategy_cat, fill_value=const_fill_cat)
            cat_data_imputed = pd.DataFrame(imp_cat.fit_transform(cat_data), columns=cat_data.columns)
        else:
            cat_data_imputed = cat_data
            status.markdown("#### Status: Imputing missing values...")
        return pd.concat([num_data_imputed, cat_data_imputed], axis=1)